@@ -1,4 +1,4 @@
-/* $Id: do_packets.c,v 1.53 2004/05/15 21:14:56 aturner Exp $ */
+/* $Id: do_packets.c 882 2004-11-07 04:16:26Z aturner $ */
 
 /*
  * Copyright (c) 2001-2004 Aaron Turner, Matt Bing.
@@ -30,8 +30,8 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
 
+#include "config.h"
 #include <libnet.h>
 #ifdef HAVE_PCAPNAV
 #include <pcapnav.h>
@@ -56,6 +56,7 @@
 #include "tcpreplay.h"
 #include "tcpdump.h"
 #include "cidr.h"
+#include "portmap.h"
 #include "cache.h"
 #include "err.h"
 #include "do_packets.h"
@@ -66,8 +67,9 @@
 
 
 extern struct options options;
-extern char *cachedata;
+extern char *cachedata, *intf, *intf2;
 extern CIDR *cidrdata;
+extern PORTMAP *portmap_data;
 extern struct timeval begin, end;
 extern u_int64_t bytes_sent, failed, pkts_sent;
 extern u_int64_t cache_packets;
@@ -80,6 +82,8 @@ extern LIST *xX_list;
 
 extern tcpdump_t tcpdump;
 
+
+
 #ifdef DEBUG
 extern int debug;
 #endif
@@ -305,6 +309,9 @@ do_packets(pcapnav_t * pcapnav, pcap_t * pcap, u_int32_t linktype,
             if (memcmp(options.intf1_mac, NULL_MAC, ETHER_ADDR_LEN) != 0) {
                 memcpy(eth_hdr->ether_dhost, options.intf1_mac, ETHER_ADDR_LEN);
             }
+            if (memcmp(options.intf1_smac, NULL_MAC, ETHER_ADDR_LEN) != 0) {
+                memcpy(eth_hdr->ether_shost, options.intf1_smac, ETHER_ADDR_LEN);
+            }
         }
 
         /* sometimes we should not send the packet */
@@ -329,6 +336,11 @@ do_packets(pcapnav_t * pcapnav, pcap_t * pcap, u_int32_t linktype,
             }
         }
 
+        /* rewrite ports */
+        if (options.rewriteports && (ip_hdr != NULL)) {
+            needtorecalc += rewrite_ports(portmap_data, &ip_hdr);
+        }
+
         /* Untruncate packet? Only for IP packets */
         if ((options.trunc) && (ip_hdr != NULL)) {
             needtorecalc += untrunc_packet(&pkthdr, pktdata, ip_hdr, l, l2len);
@@ -365,7 +377,8 @@ do_packets(pcapnav_t * pcapnav, pcap_t * pcap, u_int32_t linktype,
             do_sleep((struct timeval *)&pkthdr.ts, &last, pkthdr.caplen);
         }
         else if (options.one_at_a_time) {
-            printf("**** Press <ENTER> to send the next packet:\n");
+            printf("**** Press <ENTER> to send the next packet out %s\n",
+                    l == options.intf1 ? intf : intf2);
             poller[0].fd = STDIN_FILENO;
             poller[0].events = POLLIN;
             poller[0].revents = 0;
